/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "reslocator.h"

using namespace plexdata::resources;

CResourceLocator::CResourceLocator()
	: m_nSearchCalls(0)
	, m_szResourceID(NULL)
	, m_szResourceName(NULL)
	, m_szResourceType(NULL)
	, m_nSearchIndex(0)
{
}

CResourceLocator::~CResourceLocator()
{
	delete[] this->m_szResourceName;
}

bool CResourceLocator::FindResourceIDByIndex( TCHAR* szModule, LPCTSTR szType, int nIndex )
{
	bool success = false;

	if ( szModule != NULL && szModule[0] != _T('\0') )
	{
		// Load given module file.
		HMODULE hModule = ::LoadLibraryEx
			( szModule
			, NULL
			, LOAD_LIBRARY_AS_DATAFILE 
			);

		// Try to find requested resource.
		success = this->FindResourceIDByIndex( hModule, szType, nIndex );

		// Unload given module file.
		::FreeLibrary( hModule ); // NULL handle is possible!
	}

	return success;
}

bool CResourceLocator::FindResourceIDByIndex( HMODULE hModule, LPCTSTR szType, int nIndex )
{
	bool success = false;

	if ( hModule != NULL && szType != NULL )
	{
		delete[] this->m_szResourceName;

		this->m_szResourceID   = NULL;
		this->m_szResourceName = NULL;
		this->m_szResourceType = const_cast<LPTSTR>( szType );
		this->m_nSearchIndex   = nIndex;
		this->m_nSearchCalls   = 0;

		::EnumResourceTypes
			( hModule
			, (ENUMRESTYPEPROC)CResourceLocator::EnumerateResourceTypes
			, (LONG_PTR)this
			);

		// Requested resource could be found either if the 
		// resource name or the resource ID is available.
		success = ( this->m_szResourceName != NULL || this->m_szResourceID != NULL );
	}

	return success;
}

BOOL CResourceLocator::EnumerateResourceNames( HMODULE hModule, LPCTSTR lpType, LPTSTR lpName, LONG lParam )
{
	CResourceLocator* pThis = reinterpret_cast<CResourceLocator*>( lParam );
	if ( NULL == pThis )
	{
		return FALSE; // Immediately stop searching in this error case!
	}

	// If the given index matches the number of repeated function 
	// calls than an appropriated resource ID could be found and 
	// the search operation was successful.
	if ( pThis->m_nSearchCalls == pThis->m_nSearchIndex )
	{
		// Save search result depending on its data type.
		if ( IS_INTRESOURCE( lpName ) )
		{
			// Simply save this value.
			pThis->m_szResourceID = reinterpret_cast<LPTSTR>( lpName );
		}
		else
		{
			// Otherwise an appropriated buffer has to be created to store the result.
			size_t nCount = _tcslen( lpName ) + 1;
			pThis->m_szResourceName = new TCHAR[ nCount ];
			if ( pThis->m_szResourceName != NULL )
			{
				_tcscpy_s( pThis->m_szResourceName, nCount, lpName );
			}
		}
		return FALSE; // Stop search operation.
	}
	else
	{
		// Increase the search calls counter and continue searching.
		::InterlockedIncrement( &pThis->m_nSearchCalls );
		return TRUE;
	}
}

BOOL CResourceLocator::EnumerateResourceTypes( HMODULE hModule, LPTSTR lpType, LONG lParam )
{
	CResourceLocator* pThis = reinterpret_cast<CResourceLocator*>( lParam );
	if ( NULL == pThis )
	{
		return FALSE; // Immediately stop searching in this error case!
	}

	// Check if current type matches requested resources type.
	if ( IS_INTRESOURCE( lpType ) )
	{
		if ( pThis->m_szResourceType != lpType )
		{
			// Continue searching if current type does not matches requested type.
			return TRUE;
		}
	}
	else
	{
		// If lpType is not an INT resources and m_szResourceType 
		// is an INT resources they connot be the same!
		if ( IS_INTRESOURCE( pThis->m_szResourceType ) )
		{
			return TRUE;
		}
		else if ( 0 != _tcscmp( pThis->m_szResourceType, lpType ) )
		{
			// Continue searching if current type does not matches requested type.
			return TRUE;
		}
	}

	// In this case current resource type matches requested resource type
	// and therefore it's time to start serching for requested index.
	BOOL bResult = ::EnumResourceNames
		( hModule
		, lpType
		, (ENUMRESNAMEPROC)CResourceLocator::EnumerateResourceNames
		, lParam
		);

	if ( bResult == FALSE )
	{
		return FALSE;  // Don't continue because requested resource could be found!
	}
	else
	{
		return TRUE;   // Continue searching!
	}
}

// EOF