/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "shelllink.h"

#include <tchar.h>
#include <msi.h>
#include <shlobj.h>
#include <crtdbg.h>

using namespace plexdata::shelllink;

CMsiResolver::CMsiResolver()
	: m_hMSIModule(NULL)
	, m_pMsiGetShortcutTarget(NULL)
	, m_pMsiGetComponentPath(NULL)
{
}

CMsiResolver::~CMsiResolver()
{
	if ( this->m_hMSIModule != NULL )
	{
		::FreeLibrary( this->m_hMSIModule );
	}
}

bool CMsiResolver::Initialize()
{
	this->m_hMSIModule = ::LoadLibrary( _T("msi.dll") );

	// TODO: Check loaded library's version!

	if ( this->m_hMSIModule != NULL )
	{
		// Those function require the Windows Installer 4.0 or later 
		// on Windows Vista. Windows Installer on Windows Server 2003, 
		// Windows XP, and Windows 2000!

#ifdef UNICODE
		this->m_pMsiGetShortcutTarget = reinterpret_cast<MsiGetShortcutTargetW_t>( ::GetProcAddress( this->m_hMSIModule, "MsiGetShortcutTargetW" ) );
		this->m_pMsiGetComponentPath  = reinterpret_cast<MsiGetComponentPathW_t >( ::GetProcAddress( this->m_hMSIModule, "MsiGetComponentPathW"  ) );
#else
		this->m_pMsiGetShortcutTarget = reinterpret_cast<MsiGetShortcutTargetA_t>( ::GetProcAddress( this->m_hMSIModule, "MsiGetShortcutTargetA" ) );
		this->m_pMsiGetComponentPath  = reinterpret_cast<MsiGetComponentPathA_t >( ::GetProcAddress( this->m_hMSIModule, "MsiGetComponentPathA"  ) );
#endif
	}

	return ( this->m_pMsiGetShortcutTarget != NULL && this->m_pMsiGetComponentPath != NULL );
}

bool CMsiResolver::IsProperFileType
	( __in const TCHAR* szShellLink
	)
{
	bool success = false;

	_ASSERT( szShellLink                   != NULL );
	_ASSERT( this->m_pMsiGetShortcutTarget != NULL );

	if ( szShellLink != NULL )
	{
		if ( this->m_pMsiGetShortcutTarget != NULL )
		{
			unsigned int nResult                        = 0L;
			TCHAR        szProduct[MAX_GUID_CHARS+1]    = _T("");
			TCHAR        szFeature[MAX_FEATURE_CHARS+1] = _T("");
			TCHAR        szComponent[MAX_GUID_CHARS+1]  = _T("");

			nResult = this->m_pMsiGetShortcutTarget
				( szShellLink
				, szProduct
				, szFeature
				, szComponent
				);

			success = ( nResult == ERROR_SUCCESS );

		}
	}

	return success;
}

bool CMsiResolver::GetComponentPath
	( __in    const TCHAR* szShellLink
	, __out   TCHAR*       szPath
	, __inout size_t&      nPath 
	, __in    bool         bShort/*=false*/
	)
{
	bool success = false;

	_ASSERT( szShellLink                   != NULL );
	_ASSERT( this->m_pMsiGetShortcutTarget != NULL );
	_ASSERT( this->m_pMsiGetComponentPath  != NULL );

	if ( szShellLink != NULL )
	{

		if ( this->m_pMsiGetShortcutTarget != NULL && this->m_pMsiGetComponentPath != NULL )
		{
			unsigned int nResult                        = 0L;
			TCHAR        szProduct[MAX_GUID_CHARS+1]    = _T("");
			TCHAR        szFeature[MAX_FEATURE_CHARS+1] = _T("");
			TCHAR        szComponent[MAX_GUID_CHARS+1]  = _T("");

			nResult = this->m_pMsiGetShortcutTarget
				( szShellLink
				, szProduct
				, szFeature
				, szComponent
				);

			if ( nResult == ERROR_SUCCESS )
			{
				INSTALLSTATE nState      = INSTALLSTATE_UNKNOWN;
				TCHAR        szHelper[1] = _T("");
				DWORD        dwHelper    = 0;

				// Length check!
				nState = (INSTALLSTATE)this->m_pMsiGetComponentPath
					( szProduct
					, szComponent
					, szHelper
					, &dwHelper
					);

				_ASSERT( nState == INSTALLSTATE_MOREDATA );

				if ( szPath == NULL ) // Length request
				{
					nPath   = dwHelper + 1; // Set return value.
					success = true;
				}
				else
				{
					if ( dwHelper < nPath ) // Not including termination zero!
					{
						dwHelper = (DWORD)nPath;
						nState   = (INSTALLSTATE)this->m_pMsiGetComponentPath
							( szProduct
							, szComponent
							, szPath
							, &dwHelper
							);

						if ( nState >= 0 )
						{
							if ( bShort )
							{
								dwHelper = ::GetShortPathName( szPath, szPath, (unsigned long)nPath );
							}

							nPath = dwHelper;
							success = true;
						}
					}
				}
			}
		}
	}

	return success;
}
		
CShellResolver::CShellResolver()
	: m_bUninitializeNeeded(false)
	, m_pShellLink(NULL)
	, m_pPersistFile(NULL)
{
	if ( S_OK == ::CoInitializeEx( NULL, COINIT_APARTMENTTHREADED ) )
	{
		this->m_bUninitializeNeeded = true;
	}
}

CShellResolver::~CShellResolver()
{
	if ( this->m_pPersistFile != NULL )
	{
		this->m_pPersistFile->Release();
	}

	if ( this->m_pShellLink != NULL )
	{
		this->m_pShellLink->Release();
	}

	if ( this->m_bUninitializeNeeded )
	{
		::CoUninitialize();
	}
}

bool CShellResolver::Initialize()
{
	bool    success = false;
    HRESULT hResult = S_FALSE; 

	if ( this->m_pShellLink == NULL && this->m_pPersistFile == NULL )
	{
		// Get a pointer to the IShellLink interface. 
		hResult = ::CoCreateInstance( CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, IID_IShellLink, (void**)&this->m_pShellLink );

		if ( SUCCEEDED( hResult ) ) 
		{
			// Get a pointer to the IPersistFile interface. 
			hResult = this->m_pShellLink->QueryInterface( IID_IPersistFile, (void**)&this->m_pPersistFile );
		
			if ( SUCCEEDED( hResult ) ) 
			{
				success = true;
			}
			else
			{
				this->m_pShellLink->Release();
				this->m_pShellLink = NULL;
			}
		}
	}
	else
	{
		success = true; // Already initialized.
	}

	return success;
}

bool CShellResolver::Load
	( const TCHAR* szShellLink 
	, HWND         hParent/*=NULL*/
	)
{
	bool    success = false;
	HRESULT hResult = S_FALSE; 

	_ASSERT( szShellLink != NULL );

	if ( szShellLink != NULL && this->m_pShellLink != NULL && this->m_pPersistFile != NULL )
	{
		// Load the shortcut file. 
#ifdef UNICODE
		hResult = this->m_pPersistFile->Load( szShellLink, STGM_READ );
#else
		hResult = E_FAIL; // Assume an error!
		size_t nBuffer = strlen(szShellLink)+1;
		WCHAR* pBuffer = new WCHAR[nBuffer];
		if ( pBuffer != NULL )
		{
			::MultiByteToWideChar( CP_ACP, 0, szShellLink, -1, pBuffer, (int)nBuffer ); 
			hResult = this->m_pPersistFile->Load( pBuffer, STGM_READ );
			delete[] pBuffer;
		}
#endif

		if ( SUCCEEDED( hResult ) ) 
		{
			if ( hParent != NULL ) // Try to resolve only if parent is set!
			{
				hResult = this->m_pShellLink->Resolve( hParent, 0 ); 

				if ( SUCCEEDED( hResult ) ) 
				{
					success = true;
				}
			}
			else
			{
				success = true;
			}
		}
	}

	return success;
}

bool CShellResolver::Executable
	( TCHAR* szExecutable
	, size_t nExecutable 
	, bool   bShortPath/*=false*/
	)
{
	bool    success = false;
	HRESULT hResult = S_FALSE; 

	_ASSERT( szExecutable != NULL );
	_ASSERT( nExecutable  >  0    );

	if ( szExecutable != NULL && nExecutable > 0 && this->m_pShellLink != NULL && this->m_pPersistFile != NULL )
	{
		// Get link target's executable path. 
		hResult = this->m_pShellLink->GetPath
			( szExecutable
			, (int)nExecutable
			, NULL
			, bShortPath ? SLGP_SHORTPATH : 0
			); 
		success = SUCCEEDED( hResult );
	}

	return success;
}

bool CShellResolver::Description
	( TCHAR* szDescription
	, size_t nDescription 
	)
{
	bool    success = false;
	HRESULT hResult = S_FALSE; 

	_ASSERT( szDescription != NULL );
	_ASSERT( nDescription  >  0    );

	if ( szDescription != NULL && nDescription > 0 && this->m_pShellLink != NULL && this->m_pPersistFile != NULL )
	{
		// Get link target's working directory. 
		hResult = this->m_pShellLink->GetDescription
			( szDescription
			, (int)nDescription
			);
		success = SUCCEEDED( hResult );
	}

	return success;
}

bool CShellResolver::Arguments
	( TCHAR* szArguments
	, size_t nArguments 
	)
{
	bool    success = false;
	HRESULT hResult = S_FALSE; 

	_ASSERT( szArguments != NULL );
	_ASSERT( nArguments  >  0    );

	if ( szArguments != NULL && nArguments > 0 && this->m_pShellLink != NULL && this->m_pPersistFile != NULL )
	{
        // Get link target's arguments. 
		hResult = this->m_pShellLink->GetArguments
			( szArguments
			, (int)nArguments
			);
		success = SUCCEEDED( hResult );
	}

	return success;
}

bool CShellResolver::Working
	( TCHAR* szWorking
	, size_t nWorking 
	)
{
	bool    success = false;
	HRESULT hResult = S_FALSE; 

	_ASSERT( szWorking != NULL );
	_ASSERT( nWorking  >  0    );

	if ( szWorking != NULL && nWorking > 0 && this->m_pShellLink != NULL && this->m_pPersistFile != NULL )
	{
        // Get link target's working directory. 
		hResult = this->m_pShellLink->GetWorkingDirectory
			( szWorking
			, (int)nWorking
			);
		success = SUCCEEDED( hResult );
	}

	return success;
}

bool CShellResolver::IconInfo
	( TCHAR* szIconFile
	, size_t nIconFile
	, int&   nIconID 
	)
{
	bool    success = false;
	HRESULT hResult = S_FALSE; 

	_ASSERT( szIconFile != NULL );
	_ASSERT( nIconFile  >  0    );

	if ( szIconFile != NULL && nIconFile > 0 && this->m_pShellLink != NULL && this->m_pPersistFile != NULL )
	{
		nIconID = -1L;

		// Get link target's icon location and icon ID.
		hResult = this->m_pShellLink->GetIconLocation
			( szIconFile
			, (int)nIconFile
			, &nIconID
			);

		success = SUCCEEDED( hResult );
	}

	return success;
}

bool CShellResolver::ShowCommand
	( int& nShow 
	)
{
	bool    success = false;
	HRESULT hResult = S_FALSE; 

	if ( this->m_pShellLink != NULL && this->m_pPersistFile != NULL )
	{
		nShow = -1L;

        // Get link target's showing command ID. 
		hResult = this->m_pShellLink->GetShowCmd
			( &nShow
			);
		success = SUCCEEDED( hResult );
	}

	return success;
}

// EOF