/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "winmain.h"

#include <htmlhelp.h>

#include "regdlg.h"
#include "mandlg.h"
#include "aboutdlg.h"
#include "helpdlg.h"
#include "shelllink.h"
#include "reslocator.h"

using namespace plexdata::string;
using namespace plexdata::shelllink;
using namespace plexdata::resources;

/////////////////////////////////////////////////////////////////////////////
// Make an instances of all predefined global variables.
/////////////////////////////////////////////////////////////////////////////
HICON          g_hIcon        = NULL;
HWND           g_hWindow      = NULL;
HINSTANCE      g_hInstance    = NULL;
BOOL           g_bTrayEnabled = FALSE;
NOTIFYICONDATA g_sTrayData    = { 0L, NULL, 0, 0, 0, NULL, _T("") };
bool           g_bLargeIcons  = false;
bool           g_bShortPath   = false;
bool           g_bSystemFont  = true;
bool           g_bHelpOpen    = false;
LOGFONT        g_lfMenuFont   = { -11, 0, 0, 0, FW_NORMAL, FALSE, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FW_DONTCARE, _T("Tahoma") };
HMENU          g_hLeftMenu    = NULL;
HMENU          g_hRightMenu   = NULL;
CConfig        g_Config;
CString        g_strModule;
BY_HANDLE_FILE_INFORMATION g_sIniFileInfo = {0};

/////////////////////////////////////////////////////////////////////////////
// Define some global constants.
/////////////////////////////////////////////////////////////////////////////
#define WINDOW_CLASS       _T("TrayLaunchWindowClass")
#define WINDOW_NAME        _T("TrayLaunch")
#define TRAY_ICON_TOOL_TIP _T("TrayLaunch")
#define INSTALL_RUN_KEY    _T("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run")
#define INSTALL_RUN_VAL    _T("TrayLaunchWatchDog")

/////////////////////////////////////////////////////////////////////////////
// Define menu item data structure
/////////////////////////////////////////////////////////////////////////////
typedef struct tagMENUITEMDATA
{
	// Label related settings.
	CString strLabel;
	LOGFONT lfLabel;
	// Icon related settings.
	HICON   hIcon;
	int     nWidth;
	int     nHeight;
	int     nGap;
} MENUITEMDATA, *LPMENUITEMDATA;

/////////////////////////////////////////////////////////////////////////////
// Local function definition section.
/////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK MainWndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

bool IsHelp( CString strCmdLine );
bool IsHelpfile( CString strCmdLine );
bool IsInstall( CString strCmdLine, HKEY& hWhere );
bool IsRemove( CString strCmdLine );
bool IsRun( CString strCmdLine );

void HandleLButton( HWND hWnd );
void HandleRButton( HWND hWnd );

void OnMeasureItem( HWND hWnd, WPARAM wParam, LPARAM lParam );
void OnDrawItem( HWND hWnd, WPARAM wParam, LPARAM lParam );

void OnCreate( HWND hParent, WPARAM wParam, LPARAM lParam );
void OnDestroy( HWND hParent, WPARAM wParam, LPARAM lParam );

void CreateLMenu();
void DestroyLMenu();
void CreateRMenu();
void DestroyRMenu();

bool InstallTool( HKEY hRoot );
bool RemoveTool();

bool SetConfigFile();
bool GetRecordData( CRecord* cRecord );
bool FileExists( LPCTSTR szFile );

bool NeedINIFileLoad();
void SaveINIFileInfo();
bool GetFileInfo( LPCTSTR szFileName, LPBY_HANDLE_FILE_INFORMATION pFileInfo );

bool ResourceToIndex( LPCTSTR szResource, int nResource, int& nIndex );

MENUITEMDATA* GetMenuItemData( CRecord& theRecord );

void ReadIconSettings();
void ReadFontSettings();

/******************************************************************************
** 
*******************************************************************************/
int APIENTRY _tWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow )
{
	WNDCLASSEX wcex = {0};
	MSG        msg  = {0};

	// Before doing anything we have to initialize the COM library.
	::CoInitializeEx( NULL, COINIT_APARTMENTTHREADED );

	// Save the global instance handle.
	g_hInstance = hInstance;

	// Get and save current module's file name for a global usage.
	unsigned long nSize = 1024;
	::GetModuleFileName( g_hInstance, g_strModule.CreateBuffer( nSize + 1 ), nSize );
	g_strModule.CopyBuffer();
	g_strModule.ReleaseBuffer();

	// Load default tray icon from resources.
	g_hIcon = (HICON)::LoadImage( g_hInstance, MAKEINTRESOURCE( IDI_WINMAIN ), IMAGE_ICON, 0, 0, LR_DEFAULTSIZE );

	// Display the help screen and finish, if requested.
	if ( IsHelp( lpCmdLine ) )
	{
		CHelpDialog dlg;
		dlg.Show( g_hInstance, g_hWindow );
		goto FINISH;
	}

	// Display the help screen and finish, if requested.
	if ( IsHelpfile( lpCmdLine ) )
	{
		ShowHelp( NULL );
		goto FINISH;
	}

	// Check the existence of the same program.
	if ( NULL != ::FindWindow( WINDOW_CLASS, WINDOW_NAME ) )
	{
		doError( NULL, -1, WINDOW_NAME, _T("Das Programm luft bereits und kann nicht noch mal gestartet werden!") );
		goto FINISH;
	}

	// Write registry keys and finish, if requested.

	HKEY hWhere = NULL;
	if ( IsInstall( lpCmdLine, hWhere  ) )
	{
		if ( !InstallTool( hWhere ) )
		{
			doError( NULL, ::GetLastError(), WINDOW_NAME, _T("Die Installation des Programmes schlug fehl!") );
		}
		goto FINISH;
	}

	// Delete registry keys and finish, if requested.
	if ( IsRemove( lpCmdLine ) )
	{
		if ( !RemoveTool() )
		{
			doError( NULL, ::GetLastError(), WINDOW_NAME, _T("Das Entfernen des Programmes schlug fehl!") );
		}
		goto FINISH;
	}

	// Disply an error message and finish, if the command line is unknown.
	if ( !IsRun( lpCmdLine ) )
	{
		doError( NULL, -1, WINDOW_NAME, _T("Das Programm kann so nicht gestartet werden!") ); // Obsolete because of displaying help window
		goto FINISH;
	}

	// Set the name of the configuration file.
	SetConfigFile();

	// Register the main window to receive messages.
	wcex.cbSize         = sizeof(WNDCLASSEX); 
	wcex.style			= 0;
	wcex.lpfnWndProc	= (WNDPROC)MainWndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= g_hInstance;
	wcex.hIcon			= NULL;
	wcex.hCursor		= NULL;
	wcex.hbrBackground	= NULL;
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= WINDOW_CLASS;
	wcex.hIconSm		= g_hIcon;
 
	if ( NULL == ::RegisterClassEx( &wcex ) )
	{
		doError( NULL, ::GetLastError(), WINDOW_NAME, _T("Das Fenster konnte nicht registrieret werden!") );
		goto FINISH;
	}

	// Create the main window.
	g_hWindow = ::CreateWindow(
		WINDOW_CLASS, 
		WINDOW_NAME, 
		WS_TILEDWINDOW,
		::GetSystemMetrics( SM_CXSCREEN ),
		::GetSystemMetrics( SM_CYSCREEN ),
		0,
		0,
		NULL, 
		NULL, 
		g_hInstance, 
		NULL
	);

	if ( NULL == g_hWindow )
	{
		doError( NULL, ::GetLastError(), WINDOW_NAME, _T("Das Fenster konnte nicht erzeugt werden!") );
		goto FINISH;
	}

	// Initialize tray icon data structure.
	g_sTrayData.cbSize           = sizeof(NOTIFYICONDATA);
	g_sTrayData.hWnd             = g_hWindow;
	g_sTrayData.uID              = 0;
	g_sTrayData.hIcon            = g_hIcon;
	g_sTrayData.uFlags           = NIF_MESSAGE | NIF_ICON | NIF_TIP;
	g_sTrayData.uCallbackMessage = WM_TRAYNOTIFY;

	_tcsncpy_s( g_sTrayData.szTip, sizeof(g_sTrayData.szTip)/sizeof(TCHAR), TRAY_ICON_TOOL_TIP, _tcslen( TRAY_ICON_TOOL_TIP ) );

	/////////////////////////////////////////////////////////////////////////////
    // Set the tray icon.
	// Bugfix: Sometimes during system startup Shell_NotifyIcon() returns FALSE. So the 
	//         workaround should look like following code!
	for ( int tries=10; tries > 0 && !g_bTrayEnabled; tries-- )
	{
		if ( ::Shell_NotifyIcon( NIM_ADD, &g_sTrayData ) )
		{
			g_bTrayEnabled = TRUE;
			continue;
		}
		::Sleep( 500 );
	}

	if ( FALSE == g_bTrayEnabled )
	{
		doError( NULL, ::GetLastError(), WINDOW_NAME, _T("Das Programm konnte nicht beim Notifizierungsdienst angemeldet werden!") );
		goto FINISH;
	}

	// Start the main event handler.
	while( ::GetMessage( &msg, NULL, 0, 0 ) ) 
	{
		::TranslateMessage( &msg );
		::DispatchMessage( &msg );
	}

FINISH:

    // Delete the tray icon.
	if ( FALSE != g_bTrayEnabled )
	{
		::Shell_NotifyIcon( NIM_DELETE, &g_sTrayData );
	}

    // Delete the main window.
	if ( NULL != g_hWindow )
	{
		::DestroyWindow( g_hWindow );
	}

    // Delete the main icon.
	if ( NULL != g_hIcon )
	{
		::DestroyIcon( g_hIcon );
	}

    // Unregister the main window class.
	::UnregisterClass( WINDOW_CLASS, g_hInstance );

	// Last but not least de-initialize the COM library.
	::CoUninitialize();

	// Return with process result.
	return ( (int)msg.wParam );
}

/******************************************************************************
** 
*******************************************************************************/
LRESULT CALLBACK MainWndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch ( uMsg )
	{
		case WM_HELP:
		{
			ShowHelp( hWnd );
			return 0; // Message handled!
		}
		case WM_MEASUREITEM:

			OnMeasureItem( hWnd, wParam, lParam );

			return TRUE;

		case WM_DRAWITEM:

			OnDrawItem( hWnd, wParam, lParam );

			return TRUE;

		case WM_TRAYNOTIFY:
		{
			static volatile bool bLocked = false;

			// Bring the given window to the front first.
			::SetForegroundWindow( hWnd );

			// Don't executie twice, if it is already executed.
			if ( TRUE == bLocked ) { return 0; }

			switch ( lParam )
			{
				case WM_LBUTTONUP:
				{
					// Set the lock flag.
					bLocked = true;

					// Handle request.
					HandleLButton( hWnd );

					// Reset the lock flag.
					bLocked = false;

					// Return successful.
					return 0;
				}
				case WM_RBUTTONUP:
				{
					// Set the lock flag.
					bLocked = true;

					// Handle request.
					HandleRButton( hWnd );

					// Reset the lock flag.
					bLocked = false;

					// Return successful.
					return 0;
				}
				default:
				{
					return ::DefWindowProc( hWnd, uMsg, wParam, lParam );
				}
			}
		}
		case WM_QUERYENDSESSION:
		{
			::PostQuitMessage( 0 );
			return ::DefWindowProc( hWnd, uMsg, wParam, lParam );
		}
		case WM_CREATE:
		{
			OnCreate( hWnd, wParam, lParam );
			return 0;
		}
		case WM_DESTROY:
		{
			OnDestroy( hWnd, wParam, lParam );
			return 0;
		}
		default:
		{
			return ::DefWindowProc( hWnd, uMsg, wParam, lParam );
		}
	}
}

/******************************************************************************
** 
*******************************************************************************/
bool IsHelp( CString strCmdLine )
{
	CString strOption = strCmdLine.Trim();
	return ( strOption.IsEmpty() || strOption.Equals( _T("-help") ) );
}

/******************************************************************************
** 
*******************************************************************************/
bool IsHelpfile( CString strCmdLine )
{
	return strCmdLine.Trim().Equals( _T("-helpfile") );
}

/******************************************************************************
** 
*******************************************************************************/
bool IsInstall( CString strCmdLine, HKEY& hWhere )
{
	// Cases: * If command line consists of "-install:all" than install tool into HKLM.
	//        * In all other cases install tool into HKCU.

	CString strOption;
	CString strWhere;
	size_t  nDelimiter = -1L;

	hWhere = HKEY_CURRENT_USER; // Set default location.

	nDelimiter = strCmdLine.FindFirst( _T(':') );
	strOption  = strCmdLine.Substring( 0, nDelimiter ).Trim();

	if ( strOption.Equals( _T("-install") ) )
	{
		if ( nDelimiter != -1L )
		{
			strWhere = strCmdLine.Substring( nDelimiter + 1, -1 ).Trim();

			if ( strWhere.Equals( _T("all") ) )
			{
				hWhere = HKEY_LOCAL_MACHINE; 
			}
		}
		return true; // Tool should be installed.
	}
	else
	{
		return false; // Install option was not used.
	}
}

/******************************************************************************
** 
*******************************************************************************/
bool IsRemove( CString strCmdLine )
{
	return strCmdLine.Trim().Equals( _T("-remove") );
}

/******************************************************************************
** 
*******************************************************************************/
bool IsRun( CString strCmdLine )
{
	return strCmdLine.Trim().Equals( _T("-run") );
}

/******************************************************************************
** 
*******************************************************************************/
void HandleLButton( HWND hWnd )
{
	if ( NeedINIFileLoad() )
	{
		CreateLMenu();
		CreateRMenu();
	}

	// Initialize the Right-Button-Popup-Menu.
	if ( NULL != g_hLeftMenu )
	{
		// First of all save current mouse position.
		POINT sMouse = {0};
		::GetCursorPos( &sMouse );

		// Enable or disable menu items depending on file existence.
		ULONG nCount = 0L;
		if ( g_Config.GetUnsigned( CConfig::SEC_GENERAL, CConfig::ITEM_COUNT, &nCount ) )
		{
			for ( ULONG nIndex=1L; nIndex < nCount+1; nIndex++ )
			{
				CRecord cRecord;
				cRecord.Section( CConfig::SEC_ITEM, nIndex );

				// Get data for given section from configuration file.
				if ( GetRecordData( &cRecord ) )
				{
					UINT nFlags = MF_BYCOMMAND;

					if ( !FileExists( cRecord.Executable() ) )
					{
						// This menu item should not be executed.
						nFlags |= MF_GRAYED | MF_DISABLED;
					}
					else
					{
						// This menu can be executed.
						nFlags |= MF_ENABLED;
					}

					::EnableMenuItem( g_hLeftMenu, ( nIndex + WM_EDIT_MENU ), nFlags );
				}
			}
		}

		// Show the left button popup menu.
		int nMenu = ::TrackPopupMenu(
			g_hLeftMenu, 
			TPM_RIGHTALIGN | TPM_BOTTOMALIGN | TPM_RETURNCMD | TPM_LEFTBUTTON,
			sMouse.x, 
			sMouse.y, 
			0, 
			hWnd, 
			NULL 
		);

		// Program setup menu item?
		if ( nMenu == WM_EDIT_MENU )
		{
			CManipulateDialog dlg;

			// Show main window to enable drag and drop via taskbar.
			::ShowWindow( hWnd, SW_MINIMIZE );

			if ( IDOK == dlg.Show( g_hInstance, hWnd ) )
			{
				CreateLMenu();
			}

			// Hide main window to remove it from the taskbar.
			::ShowWindow( hWnd, SW_HIDE );
		}
		// Any other menu item?
		else if ( nMenu > WM_EDIT_MENU )
		{
			// Calculate the item's number. This number is equivalent to the returned ID.
			nMenu -= WM_EDIT_MENU;

			// Reset current record data and initialize section name.
			CRecord cRecord;
			cRecord.Section( CConfig::SEC_ITEM, nMenu );

			// Get data for given section from configuration file.
			if ( GetRecordData( &cRecord ) )
			{
				// Start the process.
				DWORD dwResult = (DWORD)::ShellExecute
					( NULL
					, _T("open")
					, cRecord.Executable()
					, cRecord.Parameters()
					, cRecord.Working()
					, cRecord.ShowCommand() 
					);

				if ( 32 >= dwResult )
				{
					if ( dwResult == 0 ) { dwResult = SE_ERR_OOM; }  // See MSDN!
					doError( hWnd, dwResult, WINDOW_NAME, _T("Programm \"%s\" konnte nicht gestartet werden!"), cRecord.Name() );
				}
			}
		}
	}

	return;
}

/******************************************************************************
** 
*******************************************************************************/
void HandleRButton( HWND hWnd )
{
	if ( NeedINIFileLoad() )
	{
		CreateLMenu();
		CreateRMenu();
	}

	// Display right button click popup menu.
	if ( NULL != g_hRightMenu )
	{
		// Save the current cursor position first.
		POINT sMouse = {0};
		::GetCursorPos( &sMouse );

		// Show the right button popup menu.
		int nMenu = ::TrackPopupMenu(
			g_hRightMenu, 
			TPM_RIGHTALIGN | TPM_BOTTOMALIGN | TPM_RETURNCMD | TPM_LEFTBUTTON,
			sMouse.x, 
			sMouse.y, 
			0, 
			hWnd, 
			NULL 
		);

		// Handle the result of the Right-Button-Popup-Menu.
		if ( WM_REMOVE_MENU == nMenu )
		{
			if ( IDYES == doQuestion( hWnd, WINDOW_NAME, _T("Wollen Sie das Progamm wirklich de-registrieren?") ) )
			{
				RemoveTool();

				::PostQuitMessage( 0 );
			}
		}
		else if ( WM_EXIT_MENU == nMenu )
		{
			::PostQuitMessage( 0 );
		}
		else if ( WM_ABOUT_MENU == nMenu )
		{
			CAboutDialog dlg;
			dlg.Show( g_hInstance, g_hWindow/*::GetDesktopWindow()*/ );
		}
	}

	return;
}

/******************************************************************************
** 
*******************************************************************************/
void OnMeasureItem( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	LPMEASUREITEMSTRUCT pMeasure = reinterpret_cast<LPMEASUREITEMSTRUCT>( lParam );

	if ( pMeasure != NULL && pMeasure->CtlType == ODT_MENU )
	{
		LPMENUITEMDATA pItem = reinterpret_cast<LPMENUITEMDATA>( pMeasure->itemData );

		if ( pItem != NULL )
		{
			// Calculate needed rectangle.
			RECT  rect     = {0};
			HDC   hDC      = ::GetDC( hWnd );
			HFONT hUseFont = ::CreateFontIndirect( &pItem->lfLabel );
			HFONT hOldFont = (HFONT)::SelectObject( hDC, hUseFont );

			::DrawText( hDC, pItem->strLabel.String(), (int)pItem->strLabel.Length(), &rect, DT_CALCRECT );

			::SelectObject( hDC, hOldFont );

			::ReleaseDC( hWnd, hDC );

			// Item width consits of: GAP + IMAGE_WIDTH + 2*GAP + TEXT_WIDTH
			pMeasure->itemWidth  = pItem->nWidth + 3 * pItem->nGap + (rect.right - rect.left);

			// Item height depends either on text height or on icon height plus 2*GAP!
			if ( ( rect.bottom - rect.top ) < pItem->nHeight )
			{
				pMeasure->itemHeight = pItem->nHeight;
			}
			else
			{
				pMeasure->itemHeight = rect.bottom - rect.top;
			}
			pMeasure->itemHeight += 2 * pItem->nGap;
		}
	}
}

/******************************************************************************
** 
*******************************************************************************/
void OnDrawItem( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	LPDRAWITEMSTRUCT pDraw = reinterpret_cast<LPDRAWITEMSTRUCT>( lParam );

	if ( pDraw != NULL && pDraw->CtlType == ODT_MENU )
	{
		LPMENUITEMDATA pItem = reinterpret_cast<LPMENUITEMDATA>( pDraw->itemData );

		if ( pItem != NULL )
		{
			// Calculate menu item's text rectangle.
			RECT rcText = {0};

			rcText.left   = pDraw->rcItem.left;
			rcText.top    = pDraw->rcItem.top;
			rcText.right  = pDraw->rcItem.right;
			rcText.bottom = pDraw->rcItem.bottom;

			HFONT hUseFont = ::CreateFontIndirect( &pItem->lfLabel );
			HFONT hOldFont = (HFONT)::SelectObject( pDraw->hDC, hUseFont );

			::DrawText( pDraw->hDC, pItem->strLabel.String(), (int)pItem->strLabel.Length(), &rcText, DT_CALCRECT );

			int nXText = pItem->nGap + pItem->nWidth + 2 * pItem->nGap;
			int nYText = pDraw->rcItem.top + ( pDraw->rcItem.bottom - pDraw->rcItem.top - rcText.bottom + rcText.top ) / 2;

			// Prepare menu item text drawing.
			HBRUSH   hBackBrush  = NULL;
			COLORREF newTxtColor = 0L;
			int      oldBkMode   = 0L;
			COLORREF oldTxtColor = 0L;
			bool     bSelected = ( pDraw->itemState & ODS_SELECTED );
			bool     bDisabled = ( pDraw->itemState & ODS_DISABLED ) || ( pDraw->itemState & ODS_GRAYED );

			if ( bSelected )
			{
				hBackBrush  = ::GetSysColorBrush( COLOR_HIGHLIGHT );
				newTxtColor = ::GetSysColor( COLOR_HIGHLIGHTTEXT );
			}
			else if ( bDisabled )
			{
				hBackBrush  = ::GetSysColorBrush( COLOR_MENU );
				newTxtColor = ::GetSysColor( COLOR_GRAYTEXT );
			}
			else
			{
				hBackBrush  = ::GetSysColorBrush( COLOR_MENU );
				newTxtColor = ::GetSysColor( COLOR_MENUTEXT );
			}

			// Draw menu item text!
			oldBkMode   = ::SetBkMode( pDraw->hDC, TRANSPARENT );
			oldTxtColor = ::SetTextColor( pDraw->hDC, newTxtColor  );

			::FillRect( pDraw->hDC, &pDraw->rcItem, hBackBrush );
			::TextOut( pDraw->hDC, nXText, nYText, pItem->strLabel.String(), (int)pItem->strLabel.Length() );

			::SetTextColor( pDraw->hDC, oldTxtColor );
			::SetBkMode( pDraw->hDC, oldBkMode );

			// Calculate menu item's icon rectangle.
			RECT rcIcon = {0};
			rcIcon.top    = pDraw->rcItem.top + ( pDraw->rcItem.bottom - pDraw->rcItem.top - pItem->nHeight ) / 2;
			rcIcon.bottom = rcIcon.top + pItem->nHeight;
			rcIcon.left   = pDraw->rcItem.left + pItem->nGap;
			rcIcon.right  = pDraw->rcItem.left + pItem->nGap + pItem->nWidth;

			// Draw menu item icon!
			::DrawIconEx( pDraw->hDC, rcIcon.left, rcIcon.top, pItem->hIcon, pItem->nWidth, pItem->nHeight, 0, NULL, DI_NORMAL );

			::SelectObject( pDraw->hDC, hOldFont );
		}
	}
}

/******************************************************************************
** 
*******************************************************************************/
void OnCreate( HWND hParent, WPARAM wParam, LPARAM lParam )
{
	CreateLMenu();
	CreateRMenu();
}

/******************************************************************************
** 
*******************************************************************************/
void OnDestroy( HWND hParent, WPARAM wParam, LPARAM lParam )
{
	CloseHelp();

	DestroyLMenu();
	DestroyRMenu();
}

/******************************************************************************
** 
*******************************************************************************/
void CreateLMenu()
{
	DestroyLMenu();

	g_hLeftMenu = ::CreatePopupMenu();

	// Initialize left button click popup menu.
	if ( NULL != g_hLeftMenu  )
	{
		CRecord  cRecord;

		// Renew general configuration data.
		ReadIconSettings();
		ReadFontSettings();

		// Append configured tools at the menu.
		ULONG nCount = 0L;
		if ( g_Config.GetUnsigned( CConfig::SEC_GENERAL, CConfig::ITEM_COUNT, &nCount ) )
		{
			bool bHaveTools = false;

			for ( ULONG nIndex=1L; nIndex < nCount+1; nIndex++ )
			{
				// Reset current record data and initialize new section name.
				cRecord.Reset();
				cRecord.Section( CConfig::SEC_ITEM, nIndex );

				// Get data for given section from configuration file.
				if ( GetRecordData( &cRecord ) )
				{
					if ( NULL != cRecord.Name() )
					{
						int nMenu = nIndex + WM_EDIT_MENU;

						// Use a menu separator if leading character is a dash.
						if ( cRecord.IsSeparator() )
						{
							::AppendMenu( g_hLeftMenu, MF_SEPARATOR, 0, NULL );
						}
						else
						{
							// Function GetMenuItemData() returns NULL if system memory allocation 
							// falis. In this case an "empty" item will be added to the menu.
							::AppendMenu( g_hLeftMenu, MF_OWNERDRAW, nMenu, (LPCTSTR)GetMenuItemData( cRecord ) );
							bHaveTools = true;
						}
					}
				}
			}

			if ( bHaveTools )
			{
				::AppendMenu( g_hLeftMenu, MF_SEPARATOR, 0, NULL );
			}
		}

		// Add the general menu item too.
		int nIconIndex = 0;
		cRecord.Name( _T("Programme bearbeiten") );
		cRecord.IconFile( g_strModule.String() );
		ResourceToIndex( cRecord.IconFile(), -IDI_WINMAIN, nIconIndex );
		cRecord.IconIndex( nIconIndex );
		::AppendMenu( g_hLeftMenu, MF_OWNERDRAW, WM_EDIT_MENU, (LPCTSTR)GetMenuItemData( cRecord ) );
	}
}

/******************************************************************************
** 
*******************************************************************************/
void DestroyLMenu()
{
	if ( NULL != g_hLeftMenu )
	{
		// Cleanup: Delete unneeded memory.
		int nItems = ::GetMenuItemCount( g_hLeftMenu );
		for ( int nIndex=0; nIndex < nItems; nIndex++ )
		{
			MENUITEMINFO itemInfo = {0};
			itemInfo.cbSize = sizeof(MENUITEMINFO);
			itemInfo.fMask  = MIIM_TYPE | MIIM_DATA;
			::GetMenuItemInfo( g_hLeftMenu, nIndex, TRUE, &itemInfo );
			if ( itemInfo.fType & MFT_OWNERDRAW )
			{
				MENUITEMDATA* pItem = reinterpret_cast<MENUITEMDATA*>( itemInfo.dwItemData );

				if ( pItem != NULL )
				{
					::DestroyIcon( pItem->hIcon );
					delete pItem;
				}
			}
		}
		::DestroyMenu( g_hLeftMenu );
		g_hLeftMenu = NULL;
	}
}

/******************************************************************************
** 
*******************************************************************************/
void CreateRMenu()
{
	DestroyRMenu();

	g_hRightMenu = ::CreatePopupMenu();

	// Initialize the Right-Button-Popup-Menu.
	if ( NULL != g_hRightMenu )
	{
		CRecord cRecord;
		int nIconIndex = 0;

		// Add about menu item.
		cRecord.Name( _T("ber ...") );
		cRecord.IconFile( g_strModule.String() );
		ResourceToIndex( cRecord.IconFile(), -IDI_WINMAIN, nIconIndex );
		cRecord.IconIndex( nIconIndex );
		::AppendMenu( g_hRightMenu, MF_OWNERDRAW, WM_ABOUT_MENU, (LPCTSTR)GetMenuItemData( cRecord ) );

		// Add separator...
		::AppendMenu( g_hRightMenu, MF_SEPARATOR, 0, NULL );

		// Add uninstall menu item.
		cRecord.Name( _T("Deinstallieren") );
		cRecord.IconFile( g_strModule.String() );
		ResourceToIndex( cRecord.IconFile(), -IDI_UNINSTALL, nIconIndex );
		cRecord.IconIndex( nIconIndex );
		::AppendMenu( g_hRightMenu, MF_OWNERDRAW, WM_REMOVE_MENU, (LPCTSTR)GetMenuItemData( cRecord ) );

		// Add exit menu item.
		cRecord.Name( _T("Beenden") );
		cRecord.IconFile( g_strModule.String() );
		ResourceToIndex( cRecord.IconFile(), -IDI_EXIT, nIconIndex );
		cRecord.IconIndex( nIconIndex );
		::AppendMenu( g_hRightMenu, MF_OWNERDRAW, WM_EXIT_MENU, (LPCTSTR)GetMenuItemData( cRecord ) );
	}
}

/******************************************************************************
** 
*******************************************************************************/
void DestroyRMenu()
{
	if ( NULL != g_hRightMenu )
	{
		::DestroyMenu( g_hRightMenu );
		g_hRightMenu = NULL;
	}
}

/******************************************************************************
** 
*******************************************************************************/
bool InstallTool( HKEY hRoot )
{
	bool success = false;

	RemoveTool(); // Before installing ensure tool's single existence.

	if ( hRoot != NULL )
	{
		HKEY hKey = NULL;

		if ( ERROR_SUCCESS == ::RegOpenKeyEx( hRoot, INSTALL_RUN_KEY, 0, KEY_SET_VALUE, &hKey ) )
		{
			CString strExecute;

			strExecute.Format( _T("\"%s\" -run"), g_strModule.String() );

			success = ( ERROR_SUCCESS == 
				::RegSetValueEx
					( hKey
					, INSTALL_RUN_VAL
					, 0
					, REG_SZ
					, reinterpret_cast<const BYTE*>( strExecute.String() )
					, (DWORD)strExecute.ByteLength()
					) 
				);

			::RegCloseKey( hKey );
		}
	}
	return success;
}

/******************************************************************************
** 
*******************************************************************************/
bool RemoveTool()
{
	HKEY hKey = NULL;

	// Try to remove the key from every possible location and ignore errors!

	if ( ERROR_SUCCESS == ::RegOpenKeyEx( HKEY_LOCAL_MACHINE, INSTALL_RUN_KEY, 0, KEY_SET_VALUE, &hKey ) )
	{
		::RegDeleteValue( hKey, INSTALL_RUN_VAL );
		::RegCloseKey( hKey );
	}

	hKey = NULL; // Reset...

	if ( ERROR_SUCCESS == ::RegOpenKeyEx( HKEY_CURRENT_USER, INSTALL_RUN_KEY, 0, KEY_SET_VALUE, &hKey ) )
	{
		::RegDeleteValue( hKey, INSTALL_RUN_VAL );
		::RegCloseKey( hKey );
	}

	return true;
}

/******************************************************************************
** 
*******************************************************************************/
void ShowHelp( HWND hParent )
{
	CSplitPath splitter( g_strModule.String() );

	if ( hParent == NULL )
	{
		// Just open the help file.
		::ShellExecute( NULL, _T("open"), splitter.BuildWithExtension( _T(".chm") ), _T(""), _T(""), SW_SHOWNORMAL );
	}
	else
	{
		// Show TOC of current help file.
		if ( NULL != ::HtmlHelp( hParent, splitter.BuildWithExtension( _T(".chm") ), HH_DISPLAY_TOC, 0 ) )
		{
			g_bHelpOpen = true;
		}
	}
}

/******************************************************************************
** 
*******************************************************************************/
void CloseHelp()
{
	if ( g_bHelpOpen )
	{
		// Ensure the help window is closed!
		::HtmlHelp( NULL, NULL, HH_CLOSE_ALL, NULL );
		g_bHelpOpen = false;
	}
}

/******************************************************************************
** 
*******************************************************************************/
bool ResolveData( HWND hWnd, CString  strFilename, CRecord& theRecord )
{
	bool success = false;

	// Split given path
	CSplitPath splitter( strFilename.String() );

	// Use current file name as the record's name.
	CString strName = splitter.Filename();

	if ( strName.IsEmpty() )
	{
		theRecord.Name( _T("<empty>") );
	}
	else
	{
		theRecord.Name( strName.String() );
	}

	if ( splitter.IsExtension( _T(".lnk") ) )
	{
		TCHAR szData[INFOTIPSIZE+1] = _T("");
		int   nData = 0;

		CMsiResolver   msiResolver;
		CShellResolver shellResolver;

		if ( shellResolver.Initialize() )
		{
			if ( shellResolver.Load( strFilename.String() ) )
			{
				if ( msiResolver.Initialize() )
				{
					if ( msiResolver.IsProperFileType( strFilename.String() ) )
					{
						size_t nHelper = INFOTIPSIZE;
						success = msiResolver.GetComponentPath
							( strFilename.String()
							, szData
							, nHelper
							);
					}
				}

				if ( !success )
				{
					success = shellResolver.Executable
						( szData
						, INFOTIPSIZE
						);
				}

				if ( success )
				{
					// Reset previously set executable path.
					theRecord.Executable( szData );

					if ( shellResolver.Arguments( szData, INFOTIPSIZE ) )
					{
						theRecord.Parameters( szData );
					}

					if ( shellResolver.ShowCommand( nData ) )
					{
						// Translate to supported show commands.
						theRecord.ShowCommand( nData );
					}

					if ( shellResolver.Working( szData, INFOTIPSIZE ) )
					{
						theRecord.Working( szData );
					}

					if ( shellResolver.Description( szData, INFOTIPSIZE ) )
					{
						theRecord.Description( szData );
					}

					if ( shellResolver.IconInfo( szData, INFOTIPSIZE, nData ) )
					{
						if ( szData[0] != _T('\0') )
						{
							// Use icon file an icon index returned by the resolver.
							theRecord.IconFile( szData );
							theRecord.IconIndex( nData );
						}
						else
						{
							// Use default icon file an default icon index instead.
							theRecord.IconFile( theRecord.Executable() );
							theRecord.IconIndex( CRecord::DEFAULT_ICON_INDEX );
						}
					}
				}
			}
		}
	}
	else
	{
		// Just set the executable's file name.
		theRecord.Executable( strFilename.String() );
		
		// Not available right here.
		theRecord.Parameters( _T("") );
		
		// Use default show command.
		theRecord.ShowCommand( CRecord::DEFAULT_SHOW_COMMAND );
		
		// Assume to load an icon from the original file.
		theRecord.IconFile( strFilename.String() );
		
		// Use default icon index instead.
		theRecord.IconIndex( CRecord::DEFAULT_ICON_INDEX );

		// Not available right here.
		theRecord.Description( _T("") );

		// Take working directory name from original file name.
		theRecord.Working( splitter.BuildPathOnly() );

		// Try to resolve icon file information from the registry 
		// if given file name is not an executable file.
		if ( !splitter.IsExtension( _T(".exe") ) )
		{
			CString strResult;
			int     nResult = 0;

			theRecord.IconFile( _T("") ); // Its definitively not an EXE file!

			if ( GetRegIconFile( splitter.Extension(), strResult, nResult ) )
			{
				theRecord.IconFile( strResult.String() );
				theRecord.IconIndex( nResult );
			}
		}
		success = true;
	}

	// Here is the right place to expand existing environment 
	// variables possibly available within the file paths.
	if ( success )
	{
		CString strExpanded;
		if ( ExpandEnvironmentPath( theRecord.Executable(), strExpanded ) )
		{
			theRecord.Executable( strExpanded.String() );
		}

		// REMARK: The workiing directory should be expanded if the process is started!
		// if ( ExpandEnvironmentPath( theRecord.Working(), strExpanded ) )
		// {
		//     theRecord.Working( strExpanded.String() );
		// }

		if ( ExpandEnvironmentPath( theRecord.IconFile(), strExpanded ) )
		{
			theRecord.IconFile( strExpanded.String() );
		}
	}

	doLogging( _T("ResolveData()") );
	doLogging( _T("Section     |%s"), theRecord.Section() );
	doLogging( _T("Name        |%s"), theRecord.Name() );
	doLogging( _T("Executable  |%s"), theRecord.Executable() );
	doLogging( _T("Parameters  |%s"), theRecord.Parameters() );
	doLogging( _T("Working     |%s"), theRecord.Working() );
	doLogging( _T("Description |%s"), theRecord.Description() );
	doLogging( _T("IconFile    |%s"), theRecord.IconFile() );
	doLogging( _T("ShowCommand |%d"), theRecord.ShowCommand() );
	doLogging( _T("IconIndex   |%d"), theRecord.IconIndex() );

	return success;
}

/******************************************************************************
** 
*******************************************************************************/
bool ExpandEnvironmentPath( CString strSource, CString& strExpand )
{
	strExpand = _T(""); // Empty return value.
	
	// Get needed expanded path length.
	size_t nLength = ::ExpandEnvironmentStrings( strSource.String(), NULL, 0 );
	if ( 0 < nLength )
	{
		if ( 0 != ::ExpandEnvironmentStrings( strSource.String(), strExpand.CreateBuffer( nLength ), (DWORD)nLength ) )
		{
			strExpand.CopyBuffer();
		}
		strExpand.ReleaseBuffer();
	}

	return !strExpand.IsEmpty();
}

/******************************************************************************
** 
*******************************************************************************/
bool GetRegIconFile( CString strExtension, CString& strIconFile, int& nIconIndex )
{
	bool    success = false;
	HKEY    hKey1   = NULL;
	HKEY    hKey2   = NULL;
	LONG    nError  = 0;
	LONG    nBytes  = 0;
	CString strKey;
	CString strValue;

	nError = ::RegOpenKeyEx( HKEY_CLASSES_ROOT, strExtension.String(), 0, KEY_QUERY_VALUE, &hKey1 );

	if ( nError == ERROR_SUCCESS )
	{
		// Query needed length.
		nError = ::RegQueryValue( hKey1, NULL, NULL, &nBytes );

		if ( nError == ERROR_SUCCESS )
		{
			nError = ::RegQueryValue( hKey1, NULL, strKey.CreateBuffer( nBytes + sizeof(TCHAR) ), &nBytes );

			if ( nError == ERROR_SUCCESS )
			{
				strKey.CopyBuffer();
				strKey += _T("\\DefaultIcon");

				nError = ::RegOpenKeyEx( HKEY_CLASSES_ROOT, strKey.String(), 0, KEY_QUERY_VALUE, &hKey2 );

				if ( nError == ERROR_SUCCESS )
				{
					nBytes = 0;
					nError = ::RegQueryValue( hKey2, NULL, NULL, &nBytes );

					if ( nError != ERROR_SUCCESS )
					{
						DWORD dwType = 0;
						nError = ::RegQueryValueEx( hKey2, NULL, NULL, &dwType, NULL,(LPDWORD)&nBytes );
					}

					if ( nError == ERROR_SUCCESS )
					{
						nError = ::RegQueryValue( hKey2, NULL, strValue.CreateBuffer( nBytes + sizeof(TCHAR) ), &nBytes );

						if ( nError == ERROR_SUCCESS )
						{
							strValue.CopyBuffer();

							size_t nDelimiter = strValue.FindLast( _T(',') );

							if ( nDelimiter != -1L )
							{
								strIconFile = strValue.Substring( 0, nDelimiter );
								nIconIndex  = strValue.Substring( nDelimiter + 1, -1 ).Long();

								// IMPORTANT: Very important resource ID fix!
								//            If an icon index is a negative value, the absolute 
								//            value of that icon index has to be interpreted as 
								//            the resource identifier rather than as index. For 
								//            example, if an icon index is -3, the icon whose 
								//            resource identifier is 3 must to be loaded from 
								//            the file.
								int nMappedIndex = 0;
								if ( ResourceToIndex( strIconFile.String(), nIconIndex, nMappedIndex ) )
								{
									nIconIndex = nMappedIndex;
								}

								success = true;
							}
						}
					}

					::RegCloseKey( hKey2 );
				}
			}
		}

		::RegCloseKey( hKey1 );
	}

	return success;
}


/******************************************************************************
** 
*******************************************************************************/
bool GetRegFileType( LPCTSTR szFile, CString& strType )
{
	bool    success = false;
	HKEY    hKey1   = NULL;
	HKEY    hKey2   = NULL;
	LONG    nError  = 0;
	LONG    nBytes  = 0;
	CString strKey;
	CSplitPath splitter( szFile );

	nError = ::RegOpenKeyEx( HKEY_CLASSES_ROOT, splitter.Extension(), 0, KEY_QUERY_VALUE, &hKey1 );

	if ( nError == ERROR_SUCCESS )
	{
		// Query needed length.
		nError = ::RegQueryValue( hKey1, NULL, NULL, &nBytes );

		if ( nError == ERROR_SUCCESS )
		{
			nError = ::RegQueryValue( hKey1, NULL, strKey.CreateBuffer( nBytes + sizeof(TCHAR) ), &nBytes );

			if ( nError == ERROR_SUCCESS )
			{
				strKey.CopyBuffer();

				nError = ::RegOpenKeyEx( HKEY_CLASSES_ROOT, strKey.String(), 0, KEY_QUERY_VALUE, &hKey2 );

				if ( nError == ERROR_SUCCESS )
				{
					nBytes = 0;
					nError = ::RegQueryValue( hKey2, NULL, NULL, &nBytes );

					if ( nError != ERROR_SUCCESS )
					{
						DWORD dwType = 0;
						nError = ::RegQueryValueEx( hKey2, NULL, NULL, &dwType, NULL,(LPDWORD)&nBytes );
					}

					if ( nError == ERROR_SUCCESS )
					{
						nError = ::RegQueryValue( hKey2, NULL, strType.CreateBuffer( nBytes + sizeof(TCHAR) ), &nBytes );

						if ( nError == ERROR_SUCCESS )
						{
							// Use registry file type.
							strType.CopyBuffer();
						}

						if ( nError != ERROR_SUCCESS || strType.IsEmpty() ) 
						{
							// Use extension instead.
							strType = splitter.Extension();
							strType = strType.Substring( 1, -1 ).Upper();
						}

						success = true;
					}

					::RegCloseKey( hKey2 );
				}
			}
		}

		::RegCloseKey( hKey1 );
	}

	return success;
}


/******************************************************************************
** 
*******************************************************************************/
bool SetConfigFile()
{
	// Use current module's file name to prepare the INI file name.
	CSplitPath splitter( g_strModule.String() );

	// Put INI file's name as configuration file.
	g_Config.SetFileName( splitter.BuildWithExtension( _T(".ini") ) );

	SaveINIFileInfo();

	// TODO: Add code to load global config stuff!

	return true;
}

/******************************************************************************
** 
*******************************************************************************/
bool GetRecordData( CRecord* pRecord )
{
	TCHAR szData[INFOTIPSIZE+1] = _T("");
	long  nData = -1L;

    // Check input parameter.
	if ( NULL == pRecord )
	{
		return false;
	}

	if ( g_Config.GetString( pRecord->Section(), CConfig::ITEM_NAME, szData, INFOTIPSIZE ) )
	{
		pRecord->Name( szData );
	}

	if ( g_Config.GetString( pRecord->Section(), CConfig::ITEM_EXECUTABLE, szData, INFOTIPSIZE ) )
	{
		pRecord->Executable( szData );
	}

	if ( g_Config.GetSigned( pRecord->Section(), CConfig::ITEM_SHOWCOMMAND, &nData ) )
	{
		pRecord->ShowCommand( nData );
	}

	if ( g_Config.GetString( pRecord->Section(), CConfig::ITEM_PARAMETERS, szData, INFOTIPSIZE ) )
	{
		pRecord->Parameters( szData );
	}

	if ( g_Config.GetString( pRecord->Section(), CConfig::ITEM_WORKING, szData, INFOTIPSIZE ) )
	{
		pRecord->Working( szData );
	}

	if ( g_Config.GetString( pRecord->Section(), CConfig::ITEM_DESCRIPTION, szData, INFOTIPSIZE ) )
	{
		pRecord->Description( szData );
	}

	if ( g_Config.GetString( pRecord->Section(), CConfig::ITEM_ICONFILE, szData, INFOTIPSIZE ) )
	{
		pRecord->IconFile( szData );
	}

	if ( g_Config.GetSigned( pRecord->Section(), CConfig::ITEM_ICONINDEX, &nData ) )
	{
		pRecord->IconIndex( nData );
	}

	return true;
}

/******************************************************************************
** 
*******************************************************************************/
bool FileExists( LPCTSTR szFile )
{
	HANDLE hFile = NULL;

	// REMARK: If given file name is NULL '::CreateFile()' returns INVALID_HANDLE_VALUE!

	// Try to open given file to check its existence.
	if ( INVALID_HANDLE_VALUE != ( hFile = ::CreateFile( szFile, 0, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL ) ) )
	{
		::CloseHandle( hFile );
		return true;
	}
	else
	{
		return false;
	}
}

/******************************************************************************
** 
*******************************************************************************/
bool NeedINIFileLoad()
{
	bool reload = false;

	BY_HANDLE_FILE_INFORMATION sFileInfo = {0};
	if ( GetFileInfo( g_Config.GetFileName(), &sFileInfo  ) )
	{
		if ( sFileInfo.nFileSizeHigh                  != g_sIniFileInfo.nFileSizeHigh                  ||
			 sFileInfo.nFileSizeLow                   != g_sIniFileInfo.nFileSizeLow                   ||
			 sFileInfo.ftLastWriteTime.dwHighDateTime != g_sIniFileInfo.ftLastWriteTime.dwHighDateTime ||
			 sFileInfo.ftLastWriteTime.dwLowDateTime  != g_sIniFileInfo.ftLastWriteTime.dwLowDateTime  
		)
		{
			reload = true;  // Reload needed.
			// Save new INI file information.
			::CopyMemory( &g_sIniFileInfo, &sFileInfo, sizeof(BY_HANDLE_FILE_INFORMATION) );
		}
	}
	return reload;
}

/******************************************************************************
** 
*******************************************************************************/
void SaveINIFileInfo()
{
	BY_HANDLE_FILE_INFORMATION sFileInfo = {0};
	if ( GetFileInfo( g_Config.GetFileName(), &sFileInfo  ) )
	{
		::CopyMemory( &g_sIniFileInfo, &sFileInfo, sizeof(BY_HANDLE_FILE_INFORMATION) );
	}
}

/******************************************************************************
** 
*******************************************************************************/
bool GetFileInfo( LPCTSTR szFileName, LPBY_HANDLE_FILE_INFORMATION pFileInfo )
{
	bool success = false;

	if ( pFileInfo != NULL )
	{
		HANDLE hFile = ::CreateFile( szFileName, 0, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
		if ( INVALID_HANDLE_VALUE != hFile )
		{
			if ( ::GetFileInformationByHandle( hFile, pFileInfo ) )
			{
				success = true;
			}
			::CloseHandle( hFile );
		}
	}
	return success;
}

/******************************************************************************
** 
*******************************************************************************/
bool ResourceToIndex( LPCTSTR szResource, int nResource, int& nIndex )
{
	bool success = false;

	CResourceLocator resLocator;

	if ( nResource < 0 )
	{
		// Convert resource ID into its positive value!
		nResource = abs( nResource );

		// Load module and suppress messages if loading fails.
		HMODULE hModule = ::LoadLibraryEx
			( szResource
			, NULL
			, LOAD_LIBRARY_AS_DATAFILE 
			);

		for ( int index=0; resLocator.FindIconID( hModule, index ); index++ )
		{
			// Fits interger resource IDs only!
			if ( IS_INTRESOURCE( resLocator.GetResourceIDString() ) )
			{
				if ( nResource == (int)resLocator.GetResourceIDString() )
				{
					nIndex = index;
					success = true;
					break;
				}
			}
		}

		::FreeLibrary( hModule );
	}

	return success;
}

/******************************************************************************
** 
*******************************************************************************/
MENUITEMDATA* GetMenuItemData( CRecord& theRecord )
{
	MENUITEMDATA* pItem = new MENUITEMDATA;

	if ( pItem != NULL )
	{
		pItem->strLabel = theRecord.Name();
		pItem->nGap     = 4;
		pItem->nWidth   = ::GetSystemMetrics( g_bLargeIcons ? SM_CXICON : SM_CXSMICON );
		pItem->nHeight  = ::GetSystemMetrics( g_bLargeIcons ? SM_CYICON : SM_CYSMICON );
		pItem->hIcon    = NULL;

		::CopyMemory( &pItem->lfLabel, &g_lfMenuFont, sizeof(LOGFONT) );

		// Using system font style overwrites any conigured font style!
		if ( g_bSystemFont )
		{
			NONCLIENTMETRICS sysMetrics  = {0};
#if ( WINVER >= 0x0600 )
			sysMetrics.cbSize = sizeof(NONCLIENTMETRICS) - sizeof(sysMetrics.iPaddedBorderWidth);
#else
			sysMetrics.cbSize = sizeof(NONCLIENTMETRICS);		
#endif // WINVER >= 0x0600 

			// Under Vista it works in the same way too!
			if ( ::SystemParametersInfo( SPI_GETNONCLIENTMETRICS, sizeof(NONCLIENTMETRICS)/*sysMetrics.cbSize*/, &sysMetrics, 0 ) )
			{
				::CopyMemory( &pItem->lfLabel, &sysMetrics.lfMenuFont, sizeof(LOGFONT) );
			}
			else
			{
				_ASSERT( false );
			}
		}

		// Load module and suppress messages if loading fails.
		HMODULE hModule = ::LoadLibraryEx
			( theRecord.IconFile()
			, NULL
			, LOAD_LIBRARY_AS_DATAFILE 
			);

		CResourceLocator resLocator;
		if ( resLocator.FindIconID( hModule, theRecord.IconIndex() ) )
		{
			// Load icon from given resource file.
			pItem->hIcon = (HICON)::LoadImage
				( hModule
				, resLocator.GetResourceIDString()
				, IMAGE_ICON
				, pItem->nWidth
				, pItem->nHeight
				, 0L 
				);
		}
		else
		{
			// REMARK: Don't take care about the file extension because I've seen that ...
			//         an executable (exactly the executable ARPPRODUCTICON.EXE which is
			//         part of Nokia's PC Suite) is apparently an icon file instead of
			//         an executable. So try to load an icon from every file type and 
			//         if successful display the result.

			// Try londing an icon from given external file.
			pItem->hIcon = (HICON)::LoadImage
				( NULL
				, theRecord.IconFile()
				, IMAGE_ICON
				, pItem->nWidth
				, pItem->nHeight
				, LR_LOADFROMFILE
				);
		}

		::FreeLibrary( hModule );
	}

	return pItem;
}

/******************************************************************************
** 
*******************************************************************************/
void ReadIconSettings()
{
	// TODO: If more global data has to be read than put them here.

	CString strData;

	if ( g_Config.GetString( CConfig::SEC_GENERAL, CConfig::ICON_SIZE, strData.CreateBuffer(INFOTIPSIZE + 1) , INFOTIPSIZE ) )
	{
		strData.CopyBuffer();

		if ( strData.Equals( _T("large") ) )
		{
			g_bLargeIcons = true;
		}
		else if ( strData.Equals( _T("small") ) )
		{
			g_bLargeIcons = false;
		}
		else
		{
			// Don't change default/previous value.
		}
	}
}

/******************************************************************************
** 
*******************************************************************************/
void ReadFontSettings()
{
	bool bShortPath = false;
	if ( g_Config.GetBoolean( CConfig::SEC_GENERAL, CConfig::SHORT_PATH, &bShortPath ) )
	{
		g_bShortPath = bShortPath;
	}

	bool bSystemFont = false;
	if ( g_Config.GetBoolean( CConfig::SEC_GENERAL, CConfig::SYS_FONT, &bSystemFont ) )
	{
		g_bSystemFont = bSystemFont;
	}

	TCHAR szName[LF_FACESIZE+1] = _T("");
	if ( g_Config.GetString( CConfig::SEC_FONT, CConfig::FONT_NAME, szName, LF_FACESIZE ) )
	{
		_tcsncpy_s( g_lfMenuFont.lfFaceName, LF_FACESIZE, szName, _tcslen( szName ) );
	}

	long nSize = 0;
	if ( g_Config.GetSigned( CConfig::SEC_FONT, CConfig::FONT_SIZE, &nSize ) )
	{
		HDC hDC = ::GetDC( ::GetDesktopWindow() );
		g_lfMenuFont.lfHeight = -::MulDiv( nSize, ::GetDeviceCaps( hDC, LOGPIXELSY ), 72 );
		::ReleaseDC( ::GetDesktopWindow(), hDC );
	}

	bool bBold = false;
	if ( g_Config.GetBoolean( CConfig::SEC_FONT, CConfig::FONT_BOLD, &bBold ) )
	{
		g_lfMenuFont.lfWeight = bBold ? FW_BOLD : FW_NORMAL;
	}

	bool bItalic = false;
	if ( g_Config.GetBoolean( CConfig::SEC_FONT, CConfig::FONT_ITALIC, &bItalic ) )
	{
		g_lfMenuFont.lfItalic = bItalic ? TRUE : FALSE;
	}

	bool bUnderline = false;
	if ( g_Config.GetBoolean( CConfig::SEC_FONT, CConfig::FONT_UNDERLINE, &bUnderline ) )
	{
		g_lfMenuFont.lfUnderline = bUnderline ? TRUE : FALSE;
	}
}

// EOF